/*******************************************************************************



    VOLTIMETRO FALANTE

    Modo PWM - utiliza o PWM interno do pic a 19,53 khz.

    Autor: Claudio Lrios

    Data: 27/10/2017

    Microcontrolador: PIC 16f628a
    FLASH: 25L1606E  (usada como 'BIOS' de alguns notebook,pc,roteadores,etc)

    Objetivo: Ao apertar o boto 'Disp' comea a informar a tenso nas pontas
    de provas por meio de voz no alto falante.

    Parte integrante do blog PICSOURCE (LARIOS.TECNOLOGIA.WS)
    Proposta didtica apenas, sendo testada somente em placa de protoboard.

******************************************************************************/


#include <16F628A.h>
#use delay(clock=20000000)//USE XTAL DE 20MHZ
#fuses NOWDT,HS, NOPUT, NOPROTECT, BROWNOUT, NOMCLR, NOLVP, NOCPD
#use fast_io(a)
#use fast_io(b)
#ZERO_RAM
#rom 0x2100 = {0,0,0}

#byte trisa= 0x85
#byte trisb= 0x86
#byte porta= 0x05
#byte portb= 0x06
#byte pr2=   0x92
#byte ccp1l= 0x15
#byte cmcon = 0x1f
#byte vrcon = 0x9f
#byte timer0= 0x01
#byte option_reg = 0x81

#bit    cs           = 0x06.0  //pino 6
#bit    miso         = 0x06.4  //pino 8
#bit    mosi         = 0x06.2  //pino 7
#bit    slk          = 0x06.1  //pino 6
#bit    flagt2       = 0x0C.1  //flag do timer 2
#bit    som          = 0x06.3  //pino 9 saida de som
#bit    tris_som     = 0x86.3  //sentido
#bit    tris_sda     = 0x86.0  //rb0 sentido
#bit    sw_leitura   = 0x05.5  //pino 4    leitura da voltagem
#bit    sw_auto      = 0x05.4  //pino 3   modo automatico
#bit    t0if         = 0x0b.2  // flag de over flow do tmr0
#bit    desc         = 0x05.0 //descarga ra0
#bit    c2_out       = 0x1f.7// cmcon saida c2


//==============================================================================
//   //Estrutura para fins de deslocamento de dados
//==============================================================================
 struct to_shift {
 unsigned int8 c0;
 unsigned int8 c1;
 unsigned int8 res0;
 unsigned int8 res1;
 unsigned int8 res2;
 } res;

//variveis globais
unsigned int8 resc0,resc1;
unsigned int16 b,dd;

unsigned int8 c,dez,un,pv;
unsigned int1 flag_auto=0;
/*==============================================================================

                      ROTINAS DE COMUNICAO SPI

==============================================================================*/


/*==============================================================================
     GERA SINAL DE CLOCK PARA A FLASH
==============================================================================*/

void clock(void){ slk=1; delay_us(1);slk=0; delay_us(1);}

/*==============================================================================
    ENVIA 8 BITS PARA A FLASH
==============================================================================*/
void env8(int8 zbuf1){
  int8 a3;
  for(a3=0;a3<8;a3++){
  if(bit_test(zbuf1,7)) mosi=1; else mosi=0;
   clock();
   zbuf1<<=1;
  }
}
/*==============================================================================
     RECEBE 8 BITS DA FLASH
==============================================================================*/

int8 recebe_flash_8bits(void){
 int8 a5,b;
 b=0;
 for (a5=0;a5<8;a5++){
  shift_left(&b,1,miso); //original
 // shift_right(&b,1,miso);//teste
  clock();
 }
   return (b);
}


/*==============================================================================
     ROTINA DE REPRODUO DE SOM GRAVADO NA MEMRIA FLASH EXTERNA
==============================================================================*/

void reproduzir(int32 address){
 int8 buf;

  cs=0;
  env8(0x03);//comando de reproduzir em modo contnuo
  env8(make8(address,2));//pega endereo inicial
  env8(make8(address,1));
  env8(make8(address,0));
  tris_som=0;//port de som como sada
  for(;;){// reproduz at encontrar 0x22 no cdigo
      buf=recebe_flash_8bits(); //recebe byte
      if(buf==0x22) {/*tris_som=1; */cs=1;break;} //para no fim do arquivo
      while(!flagt2) ;//aguarda 'pwm' finalizar processo
      ccp1l=buf;//carga do valor do pwm
      flagt2=0;//zera flag tmr2
  }

  cs=1;
}

/*==============================================================================
         ROTINAS DE ENDEREAMENTO DA FLASH 25P10 PARA CADA SOM GRAVADO
==============================================================================*/

void fala_zero(){
 reproduzir(0x00);//zero
}
void fala_um(){
 reproduzir(0x1f28);//um
}

void fala_dois(){
reproduzir(0x46f51);//dois
}

void fala_tres(){
 reproduzir(0x2f95);//tres
}

void fala_quatro(){
reproduzir(0x4c2a);//quatro
}

void fala_cinco(){
reproduzir(0x4aba8);
}

void fala_seis(){
 reproduzir(0x9410);
}

void fala_sete(){
 reproduzir(0xc239);
}

void fala_oito(){
 reproduzir(0xe9d1);
}

void fala_nove(){
 reproduzir(0x11028);
 }

void fala_dez(){
 reproduzir(0x138d1);
}

void fala_onze(){
 reproduzir(0x1556c);
}

void fala_doze(){
 reproduzir(0x1777d);
}
void fala_treze(){
 reproduzir(0x1a5ae);
}

void fala_quatorze(){
 reproduzir(0x1ce55);
}
void fala_quinze(){
 reproduzir(0x2022d);
}
void fala_dezesseis(){
 reproduzir(0x22c76);
}
void fala_dezessete(){
 reproduzir(0x26e24);
}

void fala_dezoito(){
 reproduzir(0x2abf8);
}
void fala_dezenove(){
 reproduzir(0x2e559);
}

void fala_vinte(){
 reproduzir(0x31e55);
}
void fala_trinta(){
 reproduzir(0x34d41);
}
void fala_quarenta(){
 reproduzir(0x37009);
}
void fala_virgula(){
 reproduzir(0x39a36);
}
void fala_ponto(){
 reproduzir(0x3b802);
}
void fala_blog(){
 reproduzir(0x42c38);
}
void fala_volt(){
 reproduzir(0x3ce19);
}
void fala_volts(){
 reproduzir(0x3f68a);
}

void fala_e(){
 reproduzir(0x49d74);
}

void fala_modo_auto(){
 reproduzir(0x4d9de);
}
void fala_modo_manual(){
 reproduzir(0x55ad9);
}
void fala_acima_da_escala(){
 reproduzir(0x5c060);
}
void fala_voltimetro_0_40v(){
 reproduzir(0x7463e);
}






// pega um byte e o reproduz

void pega_num(int8 val){
switch (val){
    case 0: fala_zero();break;
    case 1: fala_um();break;
    case 2: fala_dois();break;
    case 3: fala_tres();break;
    case 4: fala_quatro();break;
    case 5: fala_cinco();break;//cinco
    case 6: fala_seis();break;//seis
    case 7: fala_sete();break;//sete
    case 8: fala_oito();break;//oito
    case 9: fala_nove();break;//nove
    case 10: fala_dez();break;//dez

    }
}
void pega_num_x(int8 val){
switch (val){
   // case 0: fala_zero();break;
    case 1: fala_onze();break;
    case 2: fala_doze();break;
    case 3: fala_treze();break;
    case 4: fala_quatorze();break;
    case 5: fala_quinze();break;//cinco
    case 6: fala_dezesseis();break;//seis
    case 7: fala_dezessete();break;//sete
    case 8: fala_dezoito();break;//oito
    case 9: fala_dezenove();break;//nove
   // case 10: fala_dez();break;//dez

    }
}

void pega_num_d(int8 val){
switch (val){

    case 1: fala_dez();break;
    case 2: fala_vinte();break;
    case 3: fala_trinta();break;
    case 4: fala_quarenta();break;
    }
}

void fala_resultado(){


   dez=resc1>>4;
   un=resc1&0x0f;
   pv=c=resc0>>4;

   if((dez==4)&&(un||pv)){
     fala_acima_da_escala(); //
     delay_ms(300);
     return;
   }

   if(!dez){//se dezena estiver com '0' fala unidade e ps virgula
     pega_num(un);
     delay_ms(50);
     fala_virgula();
     delay_ms(50);
     pega_num(pv);
     delay_ms(50);
     fala_volts();
   }
   else{//x -dezena  maior ou igual a 1 (podendo ser:1,2,3,4)
      if(!un){  //se for dez, vinte, trinta, quarenta
         pega_num_d(dez);
         delay_ms(50);
         fala_virgula();
         delay_ms(50);
         pega_num(pv);
         delay_ms(50);
         fala_volts();
      }
      else{//y-se for onze... trinta e nove, sem os acima
         if(dez==1){//de onze a dezenove
          pega_num_x(un);
          delay_ms(50);
          fala_virgula();
          delay_ms(50);
          pega_num(pv);
          delay_ms(50);
          fala_volts();
         }
         else{//w- de 21 a 39
          pega_num_d(dez);
          delay_ms(50);
         // fala_e();   
        //  delay_ms(50);
          pega_num(un);
          delay_ms(50);
          fala_virgula();
          delay_ms(50);
          pega_num(pv);
          delay_ms(50);
          fala_volts();
         }//else w-

      }//else y-

   }//else x-



   }
//==============================================================================
//   Rotina do conversor Delta Sigma
//   obs. usa o timer 0 para produzir tempos iguais entre ciclos
//==============================================================================
int16 get_adc_ds(){
  int16 ciclo=0x1000; //12 bits de resoluo
  int16 val=0;//zera resultado
  timer0=0;
  t0if=0;
  for(;;){

   if(c2_out){desc=1; } else {desc=0; val++;}//testa sada do comparador
   if(!--ciclo){desc=0; return(val); }//testa fim da converso adc delta-sigma
   while(!t0if);//aguarda o estouro para manter tempos iguais na converso
   t0if=0;
   }
}
//==============================================================================
//Converte hex 2 bytes para decimal 3 bytes
//==============================================================================
void bin2_dec(int16 a){
  int8  cont,aux;

  res.c0=make8(a,0);//carrega valor a converter
  res.c1=make8(a,1);
  res.res0=0;//zera resultados
  res.res1=0;
  res.res2=0;
  cont=16;//numero de vezes da converso
 for(;;){//loop continuo
  shift_left(&res,5,0);//desloca 5 bytes p/ esquerda
  if(!--cont){//testa se chegou ao fim da converso
   resc0=res.res0;//copia para o buffer de apresentao
   resc1=res.res1;
   return;//fim da converso
  }
  aux=res.res2;
  if((aux+0x03)&0x08) res.res2+=0x03;//testa bit 3
  aux=res.res2;
  if((aux+0x30)&0x80) res.res2+=0x30;//testa bit 7
   aux=res.res1;
  if((aux+0x03)&0x08) res.res1+=0x03;//testa bit 3
  aux=res.res1;
  if((aux+0x30)&0x80) res.res1+=0x30;//testa bit 7
   aux=res.res0;
  if((aux+0x03)&0x08) res.res0+=0x03;//testa bit 3
  aux=res.res0;
  if((aux+0x30)&0x80) res.res0+=0x30;//testa bit 7
 }
}

/*==============================================================================
                      ROTINA MAIN
==============================================================================*/

void main() {

   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_timer_2 ( T2_DIV_BY_1, 0, 1);
   vrcon=0b11101100;//ajusta vref para 2,5 volts
   cmcon=5;//usando um comparador apenas
   setup_ccp1(CCP_PWM);
   trisa=0b11110110;
   trisb=0b11110000;
   option_reg|=0x80; //desliga pullups
   porta=0; //desliga sadas
   portb=0;
   pr2=0xff;//periodo max
   cs=1;
   delay_ms(100);
   fala_voltimetro_0_40v();//aparelho
   delay_ms(100);
   fala_blog();//nome do blog


/*==============================================================================

                      LOOP PRINCIPAL

==============================================================================*/
 for(;;){

   if(!sw_leitura){
    bin2_dec(get_adc_ds());// obtem converso delta-sigma/ converte para hex
    fala_resultado();//reproduz resultado no alto falante
   }
   if(!sw_auto){
      flag_auto=!flag_auto;
      if(flag_auto)fala_modo_auto(); else fala_modo_manual();
     delay_ms(50);
   }

   if(flag_auto){
     bin2_dec(get_adc_ds());// obtem converso delta-sigma/ converte para hex
    fala_resultado();//reproduz resultado no alto falante
    for(dd=0;dd<200;dd++){//delay entre leituras automticas
      delay_ms(50);
       if(!sw_auto){
         flag_auto=!flag_auto;
          if(flag_auto){fala_modo_auto();break; }else {fala_modo_manual();break;}//
       }
       if(!sw_leitura){//tambm fala a tenso se for apertado leitura em modo automtico
       bin2_dec(get_adc_ds());// obtem converso delta-sigma/ converte para hex
       fala_resultado();//reproduz resultado no alto falante
   }


    }
   }

   }//for(;;)
}//main
